function [Q,x,y,u]=channelflow(l,b,h,N)

% Parameters
a=sqrt(1/4*(l-b)^2-h^2);
dx=1/(N-1);

% Grid
[xi,eta]=ndgrid(0:dx:1,0:dx:1);
x=(b/2+a*eta).*xi;
y=h*eta;

% Coefficients in transformed equation
J=h*(b/2+a*eta);
a1=a^2*xi.^2+h^2;
b1=a*xi.*(b/2+a*eta);
c1=(b/2+a*eta).^2;
e1=2*a^2*xi;

% Initialize matrix and right hand side
A=sparse(N^2,N^2);
F=zeros(N^2,1);

% Mapping between (i,j) and position in solution vector
map=zeros(N,N);
map(:)=1:N^2;

% Molecules for du/dx, d2u/dx2, d2u/dy2, and d2u/dxdy
mol_xi=[0,-1,0;0,0,0;0,1,0]/2/dx;
mol_xixi=[0,1,0;0,-2,0;0,1,0]/dx^2;
mol_etaeta=mol_xixi';
mol_xieta=[1,0,-1;0,0,0;-1,0,1]/4/dx^2;

% Interior points
for i=2:N-1
  for j=2:N-1
    % Form stencil for full equation
    mol=-1/J(i,j)^2*(a1(i,j)*mol_xixi-2*b1(i,j)*mol_xieta+ ...
                     c1(i,j)*mol_etaeta+e1(i,j)*mol_xi);
    % Insert in A
    for i1=-1:1
      for j1=-1:1
        A( map(i,j), map(i+i1,j+j1) ) = mol(i1+2,j1+2);
      end
    end
    
    % RHS is 1
    F( map(i,j) ) = 1;
  end
end

% Bottom boundary (u=0)
for i=1:N
  A( map(i,1), map(i,1) ) = 1;
  F( map(i,1) ) = 0;
end

% Right boundary (u=0)
for j=1:N
  A( map(N,j), map(N,j) ) = 1;
  F( map(N,j) ) = 0;
end

% Top boundary (du/dn=0)
for i=2:N-1
  t1=-a/(a+b/2)*xi(i,N);
  t2=1;
  A( map(i,N), map(i+1,N) ) = t1/(2*dx);
  A( map(i,N), map(i-1,N) ) = -t1/(2*dx);
  A( map(i,N), map(i,N) ) = 3*t2/(2*dx);
  A( map(i,N), map(i,N-1) ) = -4*t2/(2*dx);
  A( map(i,N), map(i,N-2) ) = t2/(2*dx);
  F( map(i,N) ) = 0;
end

% Left boundary (du/dn=0)
for j=2:N
  A( map(1,j), map(1,j) ) = -3/(2*dx);
  A( map(1,j), map(2,j) ) = 4/(2*dx);
  A( map(1,j), map(3,j) ) = -1/(2*dx);
  F( map(1,j) ) = 0;
end

% Solve
U=A\F;
u=reshape(U,N,N);

% Compute Q
I=u.*abs(J);
mid=(I(1:N-1,1:N-1)+I(2:N,1:N-1)+I(1:N-1,2:N)+I(2:N,2:N))/4;
Q=sum(mid(:))*dx^2;
